using System;
using System.Diagnostics;
using Zcu.Mve.Core;
using Zcu.Mve.Numerics;

namespace Examples
{
  /// <summary>
  /// Sample module. It sums two input numbers. 
  /// This class shows all available events. Reaction to all events is implemented.
  /// </summary>
  [Zcu.Mve.Core.ModuleInfo("Milan Frank",
    "Sample module. It sums two input numbers. Reaction to all events is implemented.",
     IconName = "Sumator.ico",
     Category = "Example")]

  [PortInfo("A", "Input number a")]
  [PortInfo("B", "Input number b")]
  [PortInfo("Sum", "Sum of numbers a + b")]  
  public class AllEventSumator : Zcu.Mve.Core.Module
  {
    private ScalarNumber sum;

    /// <summary>
    /// Creates ports and registers all events.
    /// </summary>
    public AllEventSumator()
    {      
      AddInPort("A", typeof (Examples.ScalarNumber));
      AddInPort("B", typeof (Examples.ScalarNumber));
      AddOutPort("Sum", typeof (Examples.ScalarNumber));

      sum = new ScalarNumber();
      
      this.OnSimulationStartEvent += new SimulationStartCallBack(SimStart);
      this.OnSimulationEndEvent += new SimulationEndCallBack(SimEnd);
      this.OnModuleCreateEvent += new ModuleCreateCallBack(Create);
      this.OnDataReadyEvent += new DataReadyCallBack(DataReady);      
      this.OnConnectEvent += new ConnectCallBack(Connect);
      this.OnDisconnectEvent += new DisconnectCallBack(Disconnect);
      this.OnStateChangedEvent += new StateChangedCallBack(StateChanged);
      this.OnPortsChangedEvent += new PortsChangedCallBack(PortsChanged);
    }
    
    /// <summary>
    /// Caled in case of module state change (Waiting, Updating, Running). See module map execution description.
    /// Important for GUI.
    /// </summary>
    void StateChanged(object source, ModuleState newState)
    {
      Console.WriteLine("State of module " + this.Name + " changed to " + newState);
    }

    /// <summary>
    /// Called when something is changed with ports. (Add, Remove, port type, placement) 
    /// Importat for GUI.
    /// </summary>
    void PortsChanged(object source, Port port, PortChangeType change)
    {
      Console.WriteLine("Change of port " + port.Name + ". Change type " + change);
    }

    /// <summary>
    /// Called when something is beeing disconected from an input port.
    /// </summary>
    /// <param name="source"></param>
    /// <param name="portTo"></param>
    /// <param name="allowed"></param>
    void Disconnect(object source, InPort portTo, out bool allowed) 
    {
      Console.WriteLine("Someone is trying to disconnect port " + portTo.Name + " in module " + this.Name + ".");
      allowed = true;
    }

    /// <summary>
    /// Called when something is about to connect to an input port of this module.
    /// This operation can be denied by setting allowed to false.
    /// </summary>
    /// <param name="source"></param>
    /// <param name="portFrom"></param>
    /// <param name="portTo"></param>
    /// <param name="allowed"></param>
    void Connect(object source, OutPort portFrom, InPort portTo, out bool allowed) 
    {
      Console.WriteLine("Someone from module " + portFrom.Owner.Name + " is trying to connect on port " + portTo.Name + " in module " + this.Name + ".");
      allowed = true;
    }

    /// <summary>
    /// Called when SetOutput is called in module connected to particualar port of this module.
    /// </summary>
    /// <param name="source"></param>
    /// <param name="port"></param>
    void DataReady(object source, InPort port)
    {
      Console.WriteLine("DataReady at port: " + port.Name);
    }

    /// <summary>
    /// Called directly after constructor and MapModule asociation.
    /// Thus this.MapModule is accessible.
    /// </summary>
    void Create(object source)
    {
      Console.WriteLine("Modul Sumator has been created and assigned to map.");
      // We can throw exception in case we do not want this module to be added to particular mapmodule.
      // throw new MveException("Curent module cannot be added to particular map!");
    }


    /// <summary>
    /// Called before first step of simulation.
    /// </summary>
    void SimStart(object source)
    {
      Console.WriteLine("Object : " + this.Name + " simulation started.");
    }

	  /// <summary>
	  /// Called after last step of simulation.
	  /// </summary>
    void SimEnd(object source)
    {
      Console.WriteLine("Object : " + this.Name + " simulation ended.");
    }

    /// <summary>
    /// Main method of any module. Represent the main activity.
    /// </summary>
    public override void Execute()
    {
      // Get input data
      ScalarNumber a = (ScalarNumber) GetInput("A");
      ScalarNumber b = (ScalarNumber) GetInput("B");

      // Progress bar example
      for (int i = 0; i < 100; i++)
      {
        ProgressInfo(i * 0.01f);
        System.Threading.Thread.Sleep(30);
      }

      // Check whether data are same as in previous Execute call.
      if (IsDataSame("A") && IsDataSame("B"))
      {      
        // In case all input data are same as in previous step we do not have to calculate them again.
        SetOutput("Sum", sum, true);
      }
      else
      {
        // Calculate sum.
        sum.Val = a.Val + b.Val;
        // Send data to outport.
        SetOutput("Sum", sum, false);
      }
    }
  }
}
